/**
 * Sample React Native App
 * https://github.com/facebook/react-native
 *
 * @format
 */

import React from 'react';
import type {PropsWithChildren} from 'react';

import {
  Alert,
  Button,
  SafeAreaView,
  ScrollView,
  StatusBar,
  StyleSheet,
  Text,
  TextInput,
  useColorScheme,
  TouchableOpacity,
  View,
  NativeModules,
  NativeEventEmitter,
  Platform
} from 'react-native';

const {mPush} = NativeModules;
const mPushManagerEmitter = new NativeEventEmitter(mPush);

import {
  Colors,
  ReloadInstructions,
} from 'react-native/Libraries/NewAppScreen';

type SectionProps = PropsWithChildren<{
  title: string;
}>;

function Section({children, title}: SectionProps): JSX.Element {
  const isDarkMode = useColorScheme() === 'dark';
  return (
    <View style={styles.sectionContainer}>
      <Text
        style={[
          styles.sectionTitle,
          {
            color: isDarkMode ? Colors.white : Colors.black,
          },
        ]}>
        {title}
      </Text>
      <Text
        style={[
          styles.sectionDescription,
          {
            color: isDarkMode ? Colors.light : Colors.dark,
          },
        ]}>
        {children}
      </Text>
    </View>
  );
}

function App(): JSX.Element {

  const [result, setResult] = React.useState('결과');
  const [noti, setNoti] = React.useState();

  if (Platform.OS === "android") {    // Android      
    //Android 푸시 수신 클릭 리스너
    mPushManagerEmitter.addListener('notification', (noti) => {     
      const obj = JSON.parse(noti.toString());
      setNoti(obj.payload);              
      setResult("메시지 : \n" + JSON.stringify(noti));
    });

  }
  else {    // iOS
    
    // iOS 지연발송 관련
    mPush.setEnabled();

    // iOS 푸시 메시지 수신 클릭 리스너
    mPushManagerEmitter.addListener('oniOSReceiveNotification', (noti) => {
      setNoti(noti.payload);                       
      setResult("메시지 : \n" + JSON.stringify(noti));
    });
  }


  const isDarkMode = useColorScheme() === 'dark';
  
  const backgroundStyle = {
    backgroundColor: isDarkMode ? Colors.darker : Colors.lighter,
  };

  const registerService = () => {
    mPush.registerService(
      (callback :any) => {
        setResult("registerService result \n\n"+callback);
      });
  }
  
  const registerUser = () => {
    mPush.registerUser({
      cuid:"tester",
      name:"testerName"
    },
    (callback :any) => {
      setResult("registerUser result \n\n"+callback);
    });
  }

  const registerServiceAndUser = () => {
    mPush.registerServiceAndUser({
      cuid:"tester",
      name:"testerName"
    },
    (callback :any) => {
      setResult("registerServiceAndUser result \n\n"+callback);
    });
  }

  const unRegisterService = () => {
    mPush.unregisterService(
      (callback :any) => {
        setResult("unRegisterService result \n\n "+callback);
      }
    );
  }

  const unRegisterUser = () => {
    mPush.unregisterUser({
      cuid:"tester",
      cname:"testerName"
    },
      (callback :any) => {
        setResult("unRegisterUser result \n\n "+callback);
      }
    );
  }

  const readNotification = () => {
    if(noti)
    {
      mPush.readNotification({
        notification: noti
      },
      (callback :any) => {
        setResult("readNotification result \n\n "+callback);
        }
      );
    }
    else
    {
      setResult("메시지를 수신후 테스트하세요");
    }
  }

  const remoteSetBadge = () => {
    mPush.remoteSetBadge({
      badge:10
    },
    (callback :any) => {
        setResult("remoteSetBadge result \n\n "+callback);
    });
  }

  const sendNotification = () => {
    mPush.sendNotification({
      cuid : 'tester',
      serviceCode: 'ALL',
      message:'기본알림입니다.'
    },
    (callback :any) => {
        setResult("sendNotification result \n\n "+callback);
      }
    );
  }
  

  const notificationCenterGetBadge = () => {
    mPush.notificationCenterGetBadgeNumber(
      (callback :any) => {
        setResult("notificationCenterGetBadge = " + callback);  
      }
    );
  }

  const notificationCenterSetBadge = () => {
    mPush.notificationCenterSetBadgeNumber({
      badge:10
    },
    (callback :any) => {
      setResult("notificationCenterSetBadge = " + callback);  
    });
  }

  const registerGroup = () => {
    mPush.registerGroup({
      groupNum:"10"
    },
    (callback :any) => {
        setResult("registerGroup result \n\n "+callback);
    });
  }

  const unregisterGroup = () => {
    mPush.unregisterGroup({
      groupNum:"10"
    },
    (callback :any) => {
        setResult("unregisterGroup result \n\n "+callback);
    });
  }


  return (
    <SafeAreaView style={backgroundStyle}>
      <StatusBar
        barStyle={isDarkMode ? 'light-content' : 'dark-content'}
        backgroundColor={backgroundStyle.backgroundColor}
      />
      <Text style={styles.title}>React MPush Sample</Text>
      <TextInput
        multiline
        style={styles.result}
        value={result}
      />
      <ScrollView
        contentInsetAdjustmentBehavior="automatic"
        style={styles.scrollview}>
        <View
          style={{
            backgroundColor: isDarkMode ? Colors.black : Colors.white,
          }}>
            <Section title="1. 서버에 사용자 등록을 수행">
            <Button
                title="registerServiceAndUser"
                onPress={registerServiceAndUser}
            />
            <View style ={{height : '10%'}}/>
            </Section>

            <Section title="2. 테스트 메시지 발송">
            <Button
                title="sendNotification"
                onPress={sendNotification}
            />
            <View style ={{height : '10%'}}/>
            </Section>

            <Section title="3. 서버에 메시지 확인했음을 전송">
            <Button
                title="readNotification"
                onPress={readNotification}
            />
            <View style ={{height : '10%'}}/>
            </Section>

            <Section title="4. 등록된 사용자 해제">
            <Button
                title="unRegisterService"
                onPress={unRegisterService}
            />
            <View style ={{height : '10%'}}/>
            </Section>


            <Section title="서버에서 관리되는 뱃지값 변경 (iOS Only)">
            <Button
                title="remoteSetBadge"
                onPress={remoteSetBadge}
                />        
            <View style ={{height : '10%'}}/>
            </Section>

            <Section title="클라이언트 뱃지값 조회 (iOS Only)">
            <Button
                title="notificationCenterGetBadge"
                onPress={notificationCenterGetBadge}
                />   
            <View style ={{height : '10%'}}/>
            </Section>

            <Section title="클라이언트 뱃지값 설정 (iOS Only)">
            <Button
                title="notificationCenterSetBadge"
                onPress={notificationCenterSetBadge}
                />   
            <View style ={{height : '10%'}}/>
            </Section>
        </View>
      </ScrollView>
    </SafeAreaView>
  );
}

const styles = StyleSheet.create({
  title: {
    height:'5%',
    fontSize :20,
    padding:10,
    backgroundColor: Colors.white,
    textAlign:"center",
  },
  sectionContainer: {
    marginTop: 32,
    paddingHorizontal: 24,
  },
  sectionTitle: {
    fontSize: 18,
    fontWeight: '600',
  },
  sectionDescription: {
    marginTop: 8,
    fontSize: 3,
    fontWeight: '400',
  },
  highlight: {
    fontWeight: '700',
  },
  scrollview:{
    height:'75%'
  },
  result: {
    height: '20%',
    margin: 12,
    borderWidth: 1,
    padding: 10,
    backgroundColor: Colors.white,
  },
});



export default App;
